#include "main.h"
#include "i2c.h"

//	Initialisiert die I2C-Schnittstelle
//	Die Beiden Leitungen sind auf "Lesen" geschaltet und werden durch die PullUps auf HIGH gehalten
void i2c_init()
{
	clear_I2C_SCL_dir;	//	SCL: Lesen
	set_I2C_SCL;		//	SCL: HIGH (PullUp)

	clear_I2C_SDA_dir;	//	SDA: Lesen
	set_I2C_SDA;		//	SDA: HIGH(PullUp)
	_delay_us(I2C_Delay_us);
}

//	Startbedingung
//	SCL HIGH, SDA wechselt auf LOW
void i2c_start()
{
	//	SCL ist immer noch HIGH
	clear_I2C_SDA;		//	SDA: LOW
	set_I2C_SDA_dir;	//	SDA: Schreiben
	_delay_us(I2C_Delay_us);

	clear_I2C_SCL;		//	SCL: LOW
	set_I2C_SCL_dir;	//	SCL: Schreiben
	_delay_us(I2C_Delay_us);
}

//	Stopbedingung
void i2c_stop()
{
	//	SCL ist immer noch LOW

	clear_I2C_SDA;		//	SDA: LOW
	set_I2C_SDA_dir;	//	SDA: Schreiben
	_delay_us(I2C_Delay_us);

	clear_I2C_SCL_dir;	//	SCL: Lesen
	set_I2C_SCL;		//	SCL: HIGH (PullUp)
	_delay_us(I2C_Delay_us);

	clear_I2C_SDA_dir;	//	SDA: Lesen
	set_I2C_SDA;		//	SDA: HIGH (PullUp)
}

void i2c_write(const prog_char *str, unsigned char msb, unsigned char lsb, int len)
{
	i2c_init();
	i2c_start();
	i2c_put_byte(I2C_Device_ID);	//	Device ID	(Schreiben: + 0)
	i2c_put_byte(msb);				//	MSB
	i2c_put_byte(lsb);				//	LSB

	unsigned char bytes;

	for(bytes = len; bytes > 0; bytes--)
	{
		char byte = (char) pgm_read_byte(str);
		i2c_put_byte(byte);			//	Byte
		str++;
	}

	i2c_stop();
}

void i2c_put_byte(unsigned char byte)
{
	unsigned char bit;

	for(bit = 0; bit < 8; bit++)
	{

		if (byte & 0b10000000)
		{
			//	Bit = 1
			clear_I2C_SDA_dir;	//	SDA: Lesen
			set_I2C_SDA;		//	SDA: HIGH (PullUp)
		} else {
			//	Bit = 0
			clear_I2C_SDA;		//	SDA: LOW
			set_I2C_SDA_dir;	//	SDA: Schreiben
		}

		clear_I2C_SCL_dir;	//	SCL: Lesen
		set_I2C_SCL;		//	SCL: HIGH (PullUp)
		_delay_us(I2C_Delay_us);

		clear_I2C_SCL;			//	SCL: LOW
		set_I2C_SCL_dir;		//	SCL: Schreiben
		_delay_us(I2C_Delay_us);

		byte <<= 1;
	}

	//	SDA Release
	clear_I2C_SDA_dir;		//	SDA: Lesen
	set_I2C_SDA;			//	SDA: HIGH (PullUp)
	_delay_us(I2C_Delay_us);

	//	SCL wieder auf HIGH setzen
	clear_I2C_SCL_dir;	//	SCL: Lesen
	set_I2C_SCL;		//	SCL: HIGH (PullUp)
	_delay_us(I2C_Delay_us);

	//	ACK empfangen
	if (bit_is_set(I2C_SDA_r_port, I2C_SDA_bit)) print(error_i2c);
	_delay_us(I2C_Delay_us);

	clear_I2C_SCL;			//	SCL: LOW
	set_I2C_SCL_dir;		//	SCL: Schreiben
	_delay_us(I2C_Delay_us);
}

unsigned char i2c_get_byte(unsigned char ack)
{
	unsigned char bit, byte = 0;

	clear_I2C_SDA_dir;
	set_I2C_SDA;

	for(bit = 0; bit < 8; bit++)
	{
		clear_I2C_SCL_dir;
		set_I2C_SCL;
		_delay_us(I2C_Delay_us);
		if (bit_is_set(I2C_SDA_r_port, I2C_SDA_bit))
		{
			byte = (byte << 1) | 0b00000001;
		} else {
			byte = (byte << 1);
		}

		clear_I2C_SCL;
		set_I2C_SCL_dir;
		_delay_us(I2C_Delay_us);
	}

	//	ACK ausgeben?
	if (ack == true)
		clear_I2C_SDA;		//	SDA: LOW

	set_I2C_SDA_dir;		//	SDA: Schreiben
	_delay_us(I2C_Delay_us);

	clear_I2C_SCL_dir;
	set_I2C_SCL;
	_delay_us(I2C_Delay_us);

	clear_I2C_SCL;
	set_I2C_SCL_dir;
	_delay_us(I2C_Delay_us);

	clear_I2C_SDA_dir;
	set_I2C_SDA;

	return byte;
}

//	
unsigned char i2c_locate(unsigned char msb, unsigned char lsb)
{
	unsigned char byte;
	i2c_init();
	i2c_start();
	i2c_put_byte(I2C_Device_ID);		//	Device ID	(Schreiben: + 0)
	i2c_put_byte(msb);	//	MSB
	i2c_put_byte(lsb);	//	LSB
}

//	
void i2c_init_block_read() {
	i2c_init();
	i2c_start();
	i2c_put_byte(I2C_Device_ID + 1);	//	Device ID	(Lesen: + 1)
}
